/**
 * Library for the Mobile Processing project - http://mobile.processing.org
 *
 * Copyright (c) 2007 Shawn Van Every
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * @author  Shawn Van Every
 * 
 */

package com.mobvcasting.msimplelocation;

//import javax.microedition.location.Criteria;
import javax.microedition.location.Location;
import javax.microedition.location.LocationException;
import javax.microedition.location.LocationListener;
import javax.microedition.location.LocationProvider;
import javax.microedition.location.QualifiedCoordinates;

import processing.core.*;

public class MSimpleLocation implements LocationListener
{
	PMIDlet pMIDlet;

	float speed = 0.0f;
	float course = 0.0f;
	long timestamp = 0;
	double latitude = 0;
	double longitude = 0;
	float altitude = 0f;
	float horizontalAccuracy = 0f;
	float verticalAccuracy = 0f;
	
	public final static int DEFAULT_TIMEOUT = 15;
	public final static int NOT_STARTED = 0;
	public final static int STARTED_LOCATION_SEARCH = 1;
	public final static int FINISHED_LOCATION_SEARCH = 2;
	public final static int FAILED_LOCATION_SEARCH = 3;
	public final static int BAD_BAD_BAD = 4;
	
	int status = 0;
	
	public MSimpleLocation(PMIDlet parent) 
	{
		this.pMIDlet = parent;
	}
		
	public class LocationSearch implements Runnable
	{
		int timeout = DEFAULT_TIMEOUT;
		
		public LocationSearch(int _timeout)
		{
			timeout = _timeout;
		}
		
		public LocationSearch()
		{			
		}
		
		public void run()
		{
			try
			{
		    	LocationProvider provider = LocationProvider.getInstance(null);
		    	if (provider != null)
		    	{
		    		Location location = provider.getLocation(timeout);
		    		QualifiedCoordinates coordinates = location.getQualifiedCoordinates();
		    		speed = location.getSpeed();
		    		course = location.getCourse();
		    		timestamp = location.getTimestamp();
		    		altitude = coordinates.getAltitude();
		    		horizontalAccuracy = coordinates.getHorizontalAccuracy();
		    		verticalAccuracy = coordinates.getVerticalAccuracy();
		    		latitude = coordinates.getLatitude();
		    		longitude = coordinates.getLongitude();
		    		
		    		status = FINISHED_LOCATION_SEARCH;
		    		saveLastLocation(speed,course,timestamp,latitude,longitude,altitude,horizontalAccuracy,verticalAccuracy);		
		    		sendLibraryEvent();
		    	}
		    	else
		    	{
		    		status = FAILED_LOCATION_SEARCH;
		    		getLastLocation();
		    		sendLibraryEvent();
		    	}
			}
			catch(LocationException le)
			{
				// LocationProviders are currently out of service
				status = FAILED_LOCATION_SEARCH;
	    		getLastLocation();
	    		sendLibraryEvent();
			}
			catch (InterruptedException e) 
			{
				status = FAILED_LOCATION_SEARCH;
				getLastLocation();	
				sendLibraryEvent();
			}
			catch (Exception e)
			{
				status = BAD_BAD_BAD;
				getLastLocation();
				sendLibraryEvent();
			}
		}
	}

	public void startLocationSearch()
	{
		startLocationSearch(DEFAULT_TIMEOUT);
	}
	
	public void startLocationSearch(int timeout)
	{
		status = STARTED_LOCATION_SEARCH;
		
		LocationSearch locationSearch = new LocationSearch(timeout);
		Thread searchThread = new Thread(locationSearch);
		searchThread.start();
	}

	public void startIntervalLocationSearch(int timeout, int interval, int maxage)
	{
		status = STARTED_LOCATION_SEARCH;
						
		// This sets QOS requirements
		// Ignoring for now
		/*
		Criteria criteria = new Criteria();
		criteria.setHorizontalAccuracy(25); // 25m
		criteria.setVerticalAccuracy(25); // 25m
		criteria.setPreferredResponseTime(Criteria.NO_REQUIREMENT);
		criteria.setPreferredPowerConsumption(Criteria.NO_REQUIREMENT);
		criteria.setCostAllowed(false); 
		criteria.setSpeedAndCourseRequired(true);
		criteria.setAltitudeRequired(true);
		criteria.setAddressInfoRequired(true);		
		*/
		
		try
		{
	    	//LocationProvider provider = LocationProvider.getInstance(criteria);
	    	LocationProvider provider = LocationProvider.getInstance(null);
	    	if (provider != null)
	    	{	    		  
	    		provider.setLocationListener(this,interval,timeout,maxage);	    		
	    	}
	    	else
	    	{
	    		status = FAILED_LOCATION_SEARCH;
	    		getLastLocation();
	    		sendLibraryEvent();
	    	}
		}
		catch(LocationException le)
		{
			// LocationProviders are currently out of service
			status = FAILED_LOCATION_SEARCH;
    		getLastLocation();
    		sendLibraryEvent();
		}
	}
	
	private void saveLastLocation(float speed, 
			float course, long timestamp, 
			double latitude, double longitude, 
			float altitude, float horizontalAccuracy, 
			float verticalAccuracy)
	{
		String sSpeed = "" + speed;
		String sCourse = "" + course;
		String sTimestamp = "" + timestamp;
		String sLatitude = "" + latitude;
		String sLongitude = "" + longitude;
		String sAltitude = "" + altitude;
		String sHorizontalAccuracy = "" + horizontalAccuracy;
		String sVerticalAccuracy = "" + verticalAccuracy;
				
		String[] locationData = new String[8];
		locationData[0] = sSpeed;
		locationData[1] = sCourse;
		locationData[2] = sTimestamp;
		locationData[3] = sLatitude;
		locationData[4] = sLongitude;
		locationData[5] = sAltitude;
		locationData[6] = sHorizontalAccuracy;
		locationData[7] = sVerticalAccuracy;
		
		pMIDlet.saveStrings("lastlocation", locationData);		
	}
	
	private void getLastLocation()
	{
		String[] locationData = pMIDlet.loadStrings("lastlocation");
		if (locationData.length == 8)
		{
			speed = Float.parseFloat(locationData[0]);
			course = Float.parseFloat(locationData[1]);
			timestamp = Long.parseLong(locationData[2]);
			latitude = Double.parseDouble(locationData[3]);
			longitude = Double.parseDouble(locationData[4]);
			altitude = Float.parseFloat(locationData[5]);
			horizontalAccuracy = Float.parseFloat(locationData[6]);
			verticalAccuracy = Float.parseFloat(locationData[7]);
		}
		else
		{
			// Using the 0 default values..  No location has been saved
			speed = 0.0f;
			course = 0.0f;
			timestamp = 0;
			latitude = 0;
			longitude = 0;
			altitude = 0f;
			horizontalAccuracy = 0f;
			verticalAccuracy = 0f;
		}
	}

	public float getSpeed()
	{
		return speed;
	}
	
	public float getCourse()
	{
		return course;
	}

	public long getTimestamp()
	{
		return timestamp;
	}
	
	public float getAltitude()
	{
		return altitude;
	}
	
	public float getHorizontalAccuracy()
	{
		return horizontalAccuracy;
	}
	
	public float getVerticalAccuracy()
	{
		return verticalAccuracy;
	}
	
	public double getLatitude()
	{
		return latitude;
	}
	
	public double getLongitude()
	{
		return longitude;
	}
		
	public int getStatus()
	{
		return status;
	}
	
	private void sendLibraryEvent()
	{
		pMIDlet.libraryEvent(this, status, this);	    				
		System.out.println(status + " " + getLongitude());
	}
	
	public void locationUpdated(LocationProvider provider, Location location)
	{
		if (provider.getState() == LocationProvider.AVAILABLE)
		{
			QualifiedCoordinates coordinates = location.getQualifiedCoordinates();
	
			speed = location.getSpeed();
			course = location.getCourse();
			timestamp = location.getTimestamp();
			altitude = coordinates.getAltitude();
			horizontalAccuracy = coordinates.getHorizontalAccuracy();
			verticalAccuracy = coordinates.getVerticalAccuracy();
			latitude = coordinates.getLatitude();
			longitude = coordinates.getLongitude();
			
			status = FINISHED_LOCATION_SEARCH;
			saveLastLocation(speed,course,timestamp,latitude,longitude,altitude,horizontalAccuracy,verticalAccuracy);		
			sendLibraryEvent();
		}
		else //(provider.getState() == LocationProvider.OUT_OF_SERVICE || provider.getState() == LocationProvider.TEMPORARILY_UNAVAILABLE)
		{
    		status = FAILED_LOCATION_SEARCH;
    		getLastLocation();
    		sendLibraryEvent();
		}
	}
	
	
	public void providerStateChanged(LocationProvider provider, int newState) 
	{
		if (status == STARTED_LOCATION_SEARCH)
		{
			if (newState == LocationProvider.OUT_OF_SERVICE || newState == LocationProvider.TEMPORARILY_UNAVAILABLE)
			{
	    		status = FAILED_LOCATION_SEARCH;
	    		getLastLocation();
	    		sendLibraryEvent();
			}
		}
	}
}